from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.source_filters import SourceFilters
from ..models.source_type import SourceType

T = TypeVar("T", bound="DocSource")


@attr.s(auto_attribs=True)
class DocSource:
    """ Document source schema. """

    source_type: SourceType
    source_filters: SourceFilters
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        source_type = self.source_type.value

        source_filters = self.source_filters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "source_type": source_type,
                "source_filters": source_filters,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        source_type = SourceType(d.pop("source_type"))

        source_filters = SourceFilters.from_dict(d.pop("source_filters"))

        doc_source = cls(
            source_type=source_type,
            source_filters=source_filters,
        )

        doc_source.additional_properties = d
        return doc_source

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="VersioningPolicyUpdate")


@attr.s(auto_attribs=True)
class VersioningPolicyUpdate:
    """ Versioning policy update schema. """

    enabled: Union[Unset, bool] = True
    max_num_versions: Union[Unset, int] = 5
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        enabled = self.enabled
        max_num_versions = self.max_num_versions

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if max_num_versions is not UNSET:
            field_dict["max_num_versions"] = max_num_versions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enabled = d.pop("enabled", UNSET)

        max_num_versions = d.pop("max_num_versions", UNSET)

        versioning_policy_update = cls(
            enabled=enabled,
            max_num_versions=max_num_versions,
        )

        versioning_policy_update.additional_properties = d
        return versioning_policy_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.content_content_out import ContentContentOut
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    folder_id: int,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
) -> Dict[str, Any]:
    url = "{}/api/v1/folders/{folder_id}/content/".format(client.base_url, folder_id=folder_id)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "limit": limit,
        "offset": offset,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[ContentContentOut, None, None, None, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = ContentContentOut.from_dict(response.json())

        return response_200
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 404:
        response_404 = None

        return response_404
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[ContentContentOut, None, None, None, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    folder_id: int,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
) -> Response[Union[ContentContentOut, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        folder_id=folder_id,
        limit=limit,
        offset=offset,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    folder_id: int,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
) -> Optional[Union[ContentContentOut, None, None, None, HTTPValidationError]]:
    """Get list of items in a folder:
    - If the user is basic, they can only access folders they has rights on.
    - If the user is orgadmin, they can access all of their own organization's folders."""

    return sync_detailed(
        client=client,
        folder_id=folder_id,
        limit=limit,
        offset=offset,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    folder_id: int,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
) -> Response[Union[ContentContentOut, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        folder_id=folder_id,
        limit=limit,
        offset=offset,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    folder_id: int,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
) -> Optional[Union[ContentContentOut, None, None, None, HTTPValidationError]]:
    """Get list of items in a folder:
    - If the user is basic, they can only access folders they has rights on.
    - If the user is orgadmin, they can access all of their own organization's folders."""

    return (
        await asyncio_detailed(
            client=client,
            folder_id=folder_id,
            limit=limit,
            offset=offset,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.group_import_report import GroupImportReport
from ..models.user_identity import UserIdentity

T = TypeVar("T", bound="ImportReport")


@attr.s(auto_attribs=True)
class ImportReport:
    """ Import report data. """

    created_users: List[UserIdentity]
    existing_users: List[UserIdentity]
    created_groups: List[GroupImportReport]
    existing_groups: List[GroupImportReport]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_users = []
        for created_users_item_data in self.created_users:
            created_users_item = created_users_item_data.to_dict()

            created_users.append(created_users_item)

        existing_users = []
        for existing_users_item_data in self.existing_users:
            existing_users_item = existing_users_item_data.to_dict()

            existing_users.append(existing_users_item)

        created_groups = []
        for created_groups_item_data in self.created_groups:
            created_groups_item = created_groups_item_data.to_dict()

            created_groups.append(created_groups_item)

        existing_groups = []
        for existing_groups_item_data in self.existing_groups:
            existing_groups_item = existing_groups_item_data.to_dict()

            existing_groups.append(existing_groups_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "created_users": created_users,
                "existing_users": existing_users,
                "created_groups": created_groups,
                "existing_groups": existing_groups,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        created_users = []
        _created_users = d.pop("created_users")
        for created_users_item_data in _created_users:
            created_users_item = UserIdentity.from_dict(created_users_item_data)

            created_users.append(created_users_item)

        existing_users = []
        _existing_users = d.pop("existing_users")
        for existing_users_item_data in _existing_users:
            existing_users_item = UserIdentity.from_dict(existing_users_item_data)

            existing_users.append(existing_users_item)

        created_groups = []
        _created_groups = d.pop("created_groups")
        for created_groups_item_data in _created_groups:
            created_groups_item = GroupImportReport.from_dict(created_groups_item_data)

            created_groups.append(created_groups_item)

        existing_groups = []
        _existing_groups = d.pop("existing_groups")
        for existing_groups_item_data in _existing_groups:
            existing_groups_item = GroupImportReport.from_dict(existing_groups_item_data)

            existing_groups.append(existing_groups_item)

        import_report = cls(
            created_users=created_users,
            existing_users=existing_users,
            created_groups=created_groups,
            existing_groups=existing_groups,
        )

        import_report.additional_properties = d
        return import_report

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="ExtractTopChunk")


@attr.s(auto_attribs=True)
class ExtractTopChunk:
    """ Top chunk schema to be returned from the API. """

    rank: int
    chunk_id: str
    page: int
    text: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        rank = self.rank
        chunk_id = self.chunk_id
        page = self.page
        text = self.text

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "rank": rank,
                "chunk_id": chunk_id,
                "page": page,
                "text": text,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        rank = d.pop("rank")

        chunk_id = d.pop("chunk_id")

        page = d.pop("page")

        text = d.pop("text")

        extract_top_chunk = cls(
            rank=rank,
            chunk_id=chunk_id,
            page=page,
            text=text,
        )

        extract_top_chunk.additional_properties = d
        return extract_top_chunk

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from io import BytesIO
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, File, Unset

T = TypeVar("T", bound="BodyPostQuestionnaireRunApiV1ExtractQuestionnaireStart_Post")


@attr.s(auto_attribs=True)
class BodyPostQuestionnaireRunApiV1ExtractQuestionnaireStart_Post:
    """  """

    excel_questions: Union[Unset, File] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        excel_questions: Union[Unset, File] = UNSET
        if not isinstance(self.excel_questions, Unset):
            excel_questions = self.excel_questions.to_tuple()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if excel_questions is not UNSET:
            field_dict["excel_questions"] = excel_questions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        excel_questions: Union[Unset, File] = UNSET
        _excel_questions = d.pop("excel_questions", UNSET)
        if not isinstance(_excel_questions, Unset):
            excel_questions = File(payload=BytesIO(_excel_questions))

        body_post_questionnaire_run_api_v1_extract_questionnaire_start_post = cls(
            excel_questions=excel_questions,
        )

        body_post_questionnaire_run_api_v1_extract_questionnaire_start_post.additional_properties = d
        return body_post_questionnaire_run_api_v1_extract_questionnaire_start_post

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.http_validation_error import HTTPValidationError
from ...models.metadata_import_out import MetadataImportOut
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    org_id: Union[Unset, int] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/metadata/import/".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "org_id": org_id,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[List[MetadataImportOut], None, None, None, None, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = MetadataImportOut.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 404:
        response_404 = None

        return response_404
    if response.status_code == 400:
        response_400 = None

        return response_400
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[List[MetadataImportOut], None, None, None, None, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    org_id: Union[Unset, int] = UNSET,
) -> Response[Union[List[MetadataImportOut], None, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        org_id=org_id,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    org_id: Union[Unset, int] = UNSET,
) -> Optional[Union[List[MetadataImportOut], None, None, None, None, HTTPValidationError]]:
    """List pending and executing metadata import tasks.

    Only orgadmins can access metadata import tasks.

    The id of the organization is inferred from the session token.
    Only orgadmins can access metadata import tasks.
    The id of the organization is guessed from the session token."""

    return sync_detailed(
        client=client,
        org_id=org_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    org_id: Union[Unset, int] = UNSET,
) -> Response[Union[List[MetadataImportOut], None, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        org_id=org_id,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    org_id: Union[Unset, int] = UNSET,
) -> Optional[Union[List[MetadataImportOut], None, None, None, None, HTTPValidationError]]:
    """List pending and executing metadata import tasks.

    Only orgadmins can access metadata import tasks.

    The id of the organization is inferred from the session token.
    Only orgadmins can access metadata import tasks.
    The id of the organization is guessed from the session token."""

    return (
        await asyncio_detailed(
            client=client,
            org_id=org_id,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.document_outline_out_outline_item import DocumentOutlineOutOutlineItem

T = TypeVar("T", bound="DocumentOutlineOut")


@attr.s(auto_attribs=True)
class DocumentOutlineOut:
    """ Document Outline schema for database input """

    outline: List[DocumentOutlineOutOutlineItem]
    version_id: int
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        outline = []
        for outline_item_data in self.outline:
            outline_item = outline_item_data.to_dict()

            outline.append(outline_item)

        version_id = self.version_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "outline": outline,
                "version_id": version_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        outline = []
        _outline = d.pop("outline")
        for outline_item_data in _outline:
            outline_item = DocumentOutlineOutOutlineItem.from_dict(outline_item_data)

            outline.append(outline_item)

        version_id = d.pop("version_id")

        document_outline_out = cls(
            outline=outline,
            version_id=version_id,
        )

        document_outline_out.additional_properties = d
        return document_outline_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.http_validation_error import HTTPValidationError
from ...types import Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    task_id: int,
) -> Dict[str, Any]:
    url = "{}/api/v1/extract/tasks/{task_id}/auto/".format(client.base_url, task_id=task_id)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[None, None, None, None, None, None, HTTPValidationError]]:
    if response.status_code == 204:
        response_204 = None

        return response_204
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 404:
        response_404 = None

        return response_404
    if response.status_code == 409:
        response_409 = None

        return response_409
    if response.status_code == 400:
        response_400 = None

        return response_400
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[None, None, None, None, None, None, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    task_id: int,
) -> Response[Union[None, None, None, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        task_id=task_id,
    )

    response = httpx.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    task_id: int,
) -> Optional[Union[None, None, None, None, None, None, HTTPValidationError]]:
    """Run the task in automatic mode.

    Only orgadmins are allowed to manage extract tasks,
    and they can only run automatic mode on their own ongoing tasks.

    Tasks can be set in automatic mode only if they are:
    - Paragraph or question extracts.
    - In manual mode."""

    return sync_detailed(
        client=client,
        task_id=task_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    task_id: int,
) -> Response[Union[None, None, None, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        task_id=task_id,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    task_id: int,
) -> Optional[Union[None, None, None, None, None, None, HTTPValidationError]]:
    """Run the task in automatic mode.

    Only orgadmins are allowed to manage extract tasks,
    and they can only run automatic mode on their own ongoing tasks.

    Tasks can be set in automatic mode only if they are:
    - Paragraph or question extracts.
    - In manual mode."""

    return (
        await asyncio_detailed(
            client=client,
            task_id=task_id,
        )
    ).parsed



import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

from ..models.notification_out_args import NotificationOutArgs
from ..models.notification_out_info import NotificationOutInfo

T = TypeVar("T", bound="NotificationOut")


@attr.s(auto_attribs=True)
class NotificationOut:
    """ Notification schema to output from GET methods. """

    user_id: int
    message: str
    code: int
    args: NotificationOutArgs
    seen: bool
    info: NotificationOutInfo
    id: int
    timestamp: datetime.datetime
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user_id = self.user_id
        message = self.message
        code = self.code
        args = self.args.to_dict()

        seen = self.seen
        info = self.info.to_dict()

        id = self.id
        timestamp = self.timestamp.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user_id": user_id,
                "message": message,
                "code": code,
                "args": args,
                "seen": seen,
                "info": info,
                "id": id,
                "timestamp": timestamp,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user_id = d.pop("user_id")

        message = d.pop("message")

        code = d.pop("code")

        args = NotificationOutArgs.from_dict(d.pop("args"))

        seen = d.pop("seen")

        info = NotificationOutInfo.from_dict(d.pop("info"))

        id = d.pop("id")

        timestamp = isoparse(d.pop("timestamp"))

        notification_out = cls(
            user_id=user_id,
            message=message,
            code=code,
            args=args,
            seen=seen,
            info=info,
            id=id,
            timestamp=timestamp,
        )

        notification_out.additional_properties = d
        return notification_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

T = TypeVar("T", bound="BodyPutMetadataDisplayApiV1MetadataDisplay_Put")


@attr.s(auto_attribs=True)
class BodyPutMetadataDisplayApiV1MetadataDisplay_Put:
    """  """

    primary_filters: List[int]
    secondary_filters: List[int]
    chunks_results: List[int]
    sorting_metadata: List[int]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        primary_filters = self.primary_filters

        secondary_filters = self.secondary_filters

        chunks_results = self.chunks_results

        sorting_metadata = self.sorting_metadata

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "primary_filters": primary_filters,
                "secondary_filters": secondary_filters,
                "chunks_results": chunks_results,
                "sorting_metadata": sorting_metadata,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        primary_filters = cast(List[int], d.pop("primary_filters"))

        secondary_filters = cast(List[int], d.pop("secondary_filters"))

        chunks_results = cast(List[int], d.pop("chunks_results"))

        sorting_metadata = cast(List[int], d.pop("sorting_metadata"))

        body_put_metadata_display_api_v1_metadata_display_put = cls(
            primary_filters=primary_filters,
            secondary_filters=secondary_filters,
            chunks_results=chunks_results,
            sorting_metadata=sorting_metadata,
        )

        body_put_metadata_display_api_v1_metadata_display_put.additional_properties = d
        return body_put_metadata_display_api_v1_metadata_display_put

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.search_query_keywords_filters import SearchQueryKeywordsFilters
from ..types import UNSET, Unset

T = TypeVar("T", bound="SearchQueryKeywords")


@attr.s(auto_attribs=True)
class SearchQueryKeywords:
    """ Schema for keyword search queries. """

    keywords: List[str]
    folder_ids: Union[Unset, List[int]] = UNSET
    output_fields: Union[Unset, List[str]] = UNSET
    filters: Union[SearchQueryKeywordsFilters, Unset] = UNSET
    keyword_op: Union[Unset, None] = UNSET
    file_ids: Union[Unset, List[int]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        keywords = self.keywords

        folder_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.folder_ids, Unset):
            folder_ids = self.folder_ids

        output_fields: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.output_fields, Unset):
            output_fields = self.output_fields

        filters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = self.filters.to_dict()

        keyword_op = None

        file_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.file_ids, Unset):
            file_ids = self.file_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "keywords": keywords,
            }
        )
        if folder_ids is not UNSET:
            field_dict["folder_ids"] = folder_ids
        if output_fields is not UNSET:
            field_dict["output_fields"] = output_fields
        if filters is not UNSET:
            field_dict["filters"] = filters
        if keyword_op is not UNSET:
            field_dict["keyword_op"] = keyword_op
        if file_ids is not UNSET:
            field_dict["file_ids"] = file_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        keywords = cast(List[str], d.pop("keywords"))

        folder_ids = cast(List[int], d.pop("folder_ids", UNSET))

        output_fields = cast(List[str], d.pop("output_fields", UNSET))

        filters: Union[SearchQueryKeywordsFilters, Unset] = UNSET
        _filters = d.pop("filters", UNSET)
        if not isinstance(_filters, Unset):
            filters = SearchQueryKeywordsFilters.from_dict(_filters)

        keyword_op = None

        file_ids = cast(List[int], d.pop("file_ids", UNSET))

        search_query_keywords = cls(
            keywords=keywords,
            folder_ids=folder_ids,
            output_fields=output_fields,
            filters=filters,
            keyword_op=keyword_op,
            file_ids=file_ids,
        )

        search_query_keywords.additional_properties = d
        return search_query_keywords

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.version import Version
from ..types import UNSET, Unset

T = TypeVar("T", bound="FileItemOutWithLatestVersion")


@attr.s(auto_attribs=True)
class FileItemOutWithLatestVersion:
    """ File item schema to output from POST method. """

    folder_id: int
    display_name: str
    id: int
    org_id: int
    name: str
    uuid: str
    created_on: datetime.datetime
    updated_on: datetime.datetime
    current_version: Version
    idx_task_id: Union[Unset, int] = UNSET
    current_version_id: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        folder_id = self.folder_id
        display_name = self.display_name
        id = self.id
        org_id = self.org_id
        name = self.name
        uuid = self.uuid
        created_on = self.created_on.isoformat()

        updated_on = self.updated_on.isoformat()

        current_version = self.current_version.to_dict()

        idx_task_id = self.idx_task_id
        current_version_id = self.current_version_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "folder_id": folder_id,
                "display_name": display_name,
                "id": id,
                "org_id": org_id,
                "name": name,
                "uuid": uuid,
                "created_on": created_on,
                "updated_on": updated_on,
                "current_version": current_version,
            }
        )
        if idx_task_id is not UNSET:
            field_dict["idx_task_id"] = idx_task_id
        if current_version_id is not UNSET:
            field_dict["current_version_id"] = current_version_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        folder_id = d.pop("folder_id")

        display_name = d.pop("display_name")

        id = d.pop("id")

        org_id = d.pop("org_id")

        name = d.pop("name")

        uuid = d.pop("uuid")

        created_on = isoparse(d.pop("created_on"))

        updated_on = isoparse(d.pop("updated_on"))

        current_version = Version.from_dict(d.pop("current_version"))

        idx_task_id = d.pop("idx_task_id", UNSET)

        current_version_id = d.pop("current_version_id", UNSET)

        file_item_out_with_latest_version = cls(
            folder_id=folder_id,
            display_name=display_name,
            id=id,
            org_id=org_id,
            name=name,
            uuid=uuid,
            created_on=created_on,
            updated_on=updated_on,
            current_version=current_version,
            idx_task_id=idx_task_id,
            current_version_id=current_version_id,
        )

        file_item_out_with_latest_version.additional_properties = d
        return file_item_out_with_latest_version

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.organization_admin_out import OrganizationAdminOut
from ..models.organization_out import OrganizationOut
from ..types import UNSET, Unset

T = TypeVar("T", bound="OrganizationCreateOut")


@attr.s(auto_attribs=True)
class OrganizationCreateOut:
    """ Organization schema to output from POST methods. """

    org: Union[OrganizationOut, Unset] = UNSET
    admin: Union[OrganizationAdminOut, Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        org: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.org, Unset):
            org = self.org.to_dict()

        admin: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.admin, Unset):
            admin = self.admin.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if org is not UNSET:
            field_dict["org"] = org
        if admin is not UNSET:
            field_dict["admin"] = admin

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        org: Union[OrganizationOut, Unset] = UNSET
        _org = d.pop("org", UNSET)
        if not isinstance(_org, Unset):
            org = OrganizationOut.from_dict(_org)

        admin: Union[OrganizationAdminOut, Unset] = UNSET
        _admin = d.pop("admin", UNSET)
        if not isinstance(_admin, Unset):
            admin = OrganizationAdminOut.from_dict(_admin)

        organization_create_out = cls(
            org=org,
            admin=admin,
        )

        organization_create_out.additional_properties = d
        return organization_create_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ExtractTaskStatus(str, Enum):
    AUTO = "auto"
    SETUP = "setup"
    MANUAL = "manual"
    DONE = "done"
    ERROR = "error"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...types import UNSET, Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    regex: str,
) -> Dict[str, Any]:
    url = "{}/api/v1/extract/revalidate/".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "regex": regex,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[None, None, None, None, None]]:
    if response.status_code == 204:
        response_204 = None

        return response_204
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 422:
        response_422 = None

        return response_422
    if response.status_code == 409:
        response_409 = None

        return response_409
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[None, None, None, None, None]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    regex: str,
) -> Response[Union[None, None, None, None, None]]:
    kwargs = _get_kwargs(
        client=client,
        regex=regex,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    regex: str,
) -> Optional[Union[None, None, None, None, None]]:
    """Validates a regex before running a value extract task.

    Regexes need to be validated because:
    - Not all operators are supported by ES
    (https://www.elastic.co/guide/en/elasticsearch/reference/current/query-dsl-regexp-query.html)
    - Of risks of DDoS by regex injection
    (https://owasp.org/www-community/attacks/Regular_expression_Denial_of_Service_-_ReDoS)

    Only orgadmins are allowed to validate regexes."""

    return sync_detailed(
        client=client,
        regex=regex,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    regex: str,
) -> Response[Union[None, None, None, None, None]]:
    kwargs = _get_kwargs(
        client=client,
        regex=regex,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    regex: str,
) -> Optional[Union[None, None, None, None, None]]:
    """Validates a regex before running a value extract task.

    Regexes need to be validated because:
    - Not all operators are supported by ES
    (https://www.elastic.co/guide/en/elasticsearch/reference/current/query-dsl-regexp-query.html)
    - Of risks of DDoS by regex injection
    (https://owasp.org/www-community/attacks/Regular_expression_Denial_of_Service_-_ReDoS)

    Only orgadmins are allowed to validate regexes."""

    return (
        await asyncio_detailed(
            client=client,
            regex=regex,
        )
    ).parsed



""" Contains some shared types for properties """
from typing import BinaryIO, Generic, MutableMapping, Optional, TextIO, Tuple, TypeVar, Union

import attr


class Unset:
    def __bool__(self) -> bool:
        return False


UNSET: Unset = Unset()


@attr.s(auto_attribs=True)
class File:
    """ Contains information for file uploads """

    payload: Union[BinaryIO, TextIO]
    file_name: Optional[str] = None
    mime_type: Optional[str] = None

    def to_tuple(self) -> Tuple[Optional[str], Union[BinaryIO, TextIO], Optional[str]]:
        """ Return a tuple representation that httpx will accept for multipart/form-data """
        return self.file_name, self.payload, self.mime_type


T = TypeVar("T")


@attr.s(auto_attribs=True)
class Response(Generic[T]):
    """ A response from an endpoint """

    status_code: int
    content: bytes
    headers: MutableMapping[str, str]
    parsed: Optional[T]


__all__ = ["File", "Response"]



from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.extract_model_create import ExtractModelCreate
from ...models.extract_model_out import ExtractModelOut
from ...models.http_validation_error import HTTPValidationError
from ...types import Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    json_body: ExtractModelCreate,
) -> Dict[str, Any]:
    url = "{}/api/v1/extract/models/".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[ExtractModelOut, None, None, None, HTTPValidationError]]:
    if response.status_code == 201:
        response_201 = ExtractModelOut.from_dict(response.json())

        return response_201
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 409:
        response_409 = None

        return response_409
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[ExtractModelOut, None, None, None, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: ExtractModelCreate,
) -> Response[Union[ExtractModelOut, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: ExtractModelCreate,
) -> Optional[Union[ExtractModelOut, None, None, None, HTTPValidationError]]:
    """Creates a new extract model.

    Only orgadmins are allowed to create extract models.
    The id of the organization is inferred from the session token.
    Two models cannot have the same name for the same organization."""

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: ExtractModelCreate,
) -> Response[Union[ExtractModelOut, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: ExtractModelCreate,
) -> Optional[Union[ExtractModelOut, None, None, None, HTTPValidationError]]:
    """Creates a new extract model.

    Only orgadmins are allowed to create extract models.
    The id of the organization is inferred from the session token.
    Two models cannot have the same name for the same organization."""

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.task_status import TaskStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="MetadataImportOut")


@attr.s(auto_attribs=True)
class MetadataImportOut:
    """ Schema representing the output from metadata import GET. """

    id: int
    org_id: int
    status: TaskStatus
    check_task_id: int
    temp_file_uuid: str
    report_id: int
    started_on: Union[Unset, datetime.datetime] = UNSET
    update_task_id: Union[Unset, int] = UNSET
    n_found_docs: Union[Unset, int] = UNSET
    n_notfound_docs: Union[Unset, int] = UNSET
    n_created_metadata: Union[Unset, int] = UNSET
    n_replaced_metadata: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        org_id = self.org_id
        status = self.status.value

        check_task_id = self.check_task_id
        temp_file_uuid = self.temp_file_uuid
        report_id = self.report_id
        started_on: Union[Unset, str] = UNSET
        if not isinstance(self.started_on, Unset):
            started_on = self.started_on.isoformat()

        update_task_id = self.update_task_id
        n_found_docs = self.n_found_docs
        n_notfound_docs = self.n_notfound_docs
        n_created_metadata = self.n_created_metadata
        n_replaced_metadata = self.n_replaced_metadata

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "org_id": org_id,
                "status": status,
                "check_task_id": check_task_id,
                "temp_file_uuid": temp_file_uuid,
                "report_id": report_id,
            }
        )
        if started_on is not UNSET:
            field_dict["started_on"] = started_on
        if update_task_id is not UNSET:
            field_dict["update_task_id"] = update_task_id
        if n_found_docs is not UNSET:
            field_dict["n_found_docs"] = n_found_docs
        if n_notfound_docs is not UNSET:
            field_dict["n_notfound_docs"] = n_notfound_docs
        if n_created_metadata is not UNSET:
            field_dict["n_created_metadata"] = n_created_metadata
        if n_replaced_metadata is not UNSET:
            field_dict["n_replaced_metadata"] = n_replaced_metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        org_id = d.pop("org_id")

        status = TaskStatus(d.pop("status"))

        check_task_id = d.pop("check_task_id")

        temp_file_uuid = d.pop("temp_file_uuid")

        report_id = d.pop("report_id")

        started_on: Union[Unset, datetime.datetime] = UNSET
        _started_on = d.pop("started_on", UNSET)
        if not isinstance(_started_on, Unset):
            started_on = isoparse(_started_on)

        update_task_id = d.pop("update_task_id", UNSET)

        n_found_docs = d.pop("n_found_docs", UNSET)

        n_notfound_docs = d.pop("n_notfound_docs", UNSET)

        n_created_metadata = d.pop("n_created_metadata", UNSET)

        n_replaced_metadata = d.pop("n_replaced_metadata", UNSET)

        metadata_import_out = cls(
            id=id,
            org_id=org_id,
            status=status,
            check_task_id=check_task_id,
            temp_file_uuid=temp_file_uuid,
            report_id=report_id,
            started_on=started_on,
            update_task_id=update_task_id,
            n_found_docs=n_found_docs,
            n_notfound_docs=n_notfound_docs,
            n_created_metadata=n_created_metadata,
            n_replaced_metadata=n_replaced_metadata,
        )

        metadata_import_out.additional_properties = d
        return metadata_import_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.questionnaire import Questionnaire
from ..models.questionnaire_model_create import QuestionnaireModelCreate
from ..types import UNSET, Unset

T = TypeVar("T", bound="QuestionnaireCreate")


@attr.s(auto_attribs=True)
class QuestionnaireCreate:
    """ Questionnaire schema to be sent on POST /questionnaire endpoint. """

    questionnaire: Questionnaire
    model: Union[QuestionnaireModelCreate, Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        questionnaire = self.questionnaire.to_dict()

        model: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.model, Unset):
            model = self.model.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "questionnaire": questionnaire,
            }
        )
        if model is not UNSET:
            field_dict["model"] = model

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        questionnaire = Questionnaire.from_dict(d.pop("questionnaire"))

        model: Union[QuestionnaireModelCreate, Unset] = UNSET
        _model = d.pop("model", UNSET)
        if not isinstance(_model, Unset):
            model = QuestionnaireModelCreate.from_dict(_model)

        questionnaire_create = cls(
            questionnaire=questionnaire,
            model=model,
        )

        questionnaire_create.additional_properties = d
        return questionnaire_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    query: str,
    suggestions: Union[Unset, int] = 10,
) -> Dict[str, Any]:
    url = "{}/api/v1/search/autocomplete/".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "query": query,
        "suggestions": suggestions,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[List[str], None, None, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = cast(List[str], response.json())

        return response_200
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[List[str], None, None, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    query: str,
    suggestions: Union[Unset, int] = 10,
) -> Response[Union[List[str], None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        query=query,
        suggestions=suggestions,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    query: str,
    suggestions: Union[Unset, int] = 10,
) -> Optional[Union[List[str], None, None, HTTPValidationError]]:
    """Suggest queries from past ones.

    The route only looks in the user's history.

    If suggestions is specified, the route returns this number of suggestions,
    otherwise the default number of returned suggestions is 10."""

    return sync_detailed(
        client=client,
        query=query,
        suggestions=suggestions,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    query: str,
    suggestions: Union[Unset, int] = 10,
) -> Response[Union[List[str], None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        query=query,
        suggestions=suggestions,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    query: str,
    suggestions: Union[Unset, int] = 10,
) -> Optional[Union[List[str], None, None, HTTPValidationError]]:
    """Suggest queries from past ones.

    The route only looks in the user's history.

    If suggestions is specified, the route returns this number of suggestions,
    otherwise the default number of returned suggestions is 10."""

    return (
        await asyncio_detailed(
            client=client,
            query=query,
            suggestions=suggestions,
        )
    ).parsed



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="BackgroundTaskOut")


@attr.s(auto_attribs=True)
class BackgroundTaskOut:
    """ Background task schema to output from GET methods. """

    user_id: int
    name: str
    task_type: str
    total_items: int
    id: int
    created_on: datetime.datetime
    processed_items: int
    total_time: float
    status: str
    done_on: Union[Unset, datetime.datetime] = UNSET
    link: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user_id = self.user_id
        name = self.name
        task_type = self.task_type
        total_items = self.total_items
        id = self.id
        created_on = self.created_on.isoformat()

        processed_items = self.processed_items
        total_time = self.total_time
        status = self.status
        done_on: Union[Unset, str] = UNSET
        if not isinstance(self.done_on, Unset):
            done_on = self.done_on.isoformat()

        link = self.link

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user_id": user_id,
                "name": name,
                "task_type": task_type,
                "total_items": total_items,
                "id": id,
                "created_on": created_on,
                "processed_items": processed_items,
                "total_time": total_time,
                "status": status,
            }
        )
        if done_on is not UNSET:
            field_dict["done_on"] = done_on
        if link is not UNSET:
            field_dict["link"] = link

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user_id = d.pop("user_id")

        name = d.pop("name")

        task_type = d.pop("task_type")

        total_items = d.pop("total_items")

        id = d.pop("id")

        created_on = isoparse(d.pop("created_on"))

        processed_items = d.pop("processed_items")

        total_time = d.pop("total_time")

        status = d.pop("status")

        done_on: Union[Unset, datetime.datetime] = UNSET
        _done_on = d.pop("done_on", UNSET)
        if not isinstance(_done_on, Unset):
            done_on = isoparse(_done_on)

        link = d.pop("link", UNSET)

        background_task_out = cls(
            user_id=user_id,
            name=name,
            task_type=task_type,
            total_items=total_items,
            id=id,
            created_on=created_on,
            processed_items=processed_items,
            total_time=total_time,
            status=status,
            done_on=done_on,
            link=link,
        )

        background_task_out.additional_properties = d
        return background_task_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.password_policy_expected import PasswordPolicyExpected
from ..models.user_out import UserOut
from ..types import UNSET, Unset

T = TypeVar("T", bound="Login")


@attr.s(auto_attribs=True)
class Login:
    """ Login schema to output from POST /login endpoint. """

    user: UserOut
    access_token: Union[Unset, str] = UNSET
    refresh_token: Union[Unset, str] = UNSET
    update_pwd_token: Union[Unset, str] = UNSET
    policy: Union[PasswordPolicyExpected, Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user = self.user.to_dict()

        access_token = self.access_token
        refresh_token = self.refresh_token
        update_pwd_token = self.update_pwd_token
        policy: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.policy, Unset):
            policy = self.policy.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user": user,
            }
        )
        if access_token is not UNSET:
            field_dict["access_token"] = access_token
        if refresh_token is not UNSET:
            field_dict["refresh_token"] = refresh_token
        if update_pwd_token is not UNSET:
            field_dict["update_pwd_token"] = update_pwd_token
        if policy is not UNSET:
            field_dict["policy"] = policy

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user = UserOut.from_dict(d.pop("user"))

        access_token = d.pop("access_token", UNSET)

        refresh_token = d.pop("refresh_token", UNSET)

        update_pwd_token = d.pop("update_pwd_token", UNSET)

        policy: Union[PasswordPolicyExpected, Unset] = UNSET
        _policy = d.pop("policy", UNSET)
        if not isinstance(_policy, Unset):
            policy = PasswordPolicyExpected.from_dict(_policy)

        login = cls(
            user=user,
            access_token=access_token,
            refresh_token=refresh_token,
            update_pwd_token=update_pwd_token,
            policy=policy,
        )

        login.additional_properties = d
        return login

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FullPasswordPolicy")


@attr.s(auto_attribs=True)
class FullPasswordPolicy:
    """ Policy schema containing constant constraints as well. """

    org_id: int
    min_length: Union[Unset, int] = 8
    expiration: Union[Unset, int] = 90
    max_similarity: Union[Unset, float] = 0.6
    max_login_attempts: Union[Unset, int] = 3
    digit: Union[Unset, bool] = True
    upper: Union[Unset, bool] = True
    lower: Union[Unset, bool] = True
    special: Union[Unset, bool] = True
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        org_id = self.org_id
        min_length = self.min_length
        expiration = self.expiration
        max_similarity = self.max_similarity
        max_login_attempts = self.max_login_attempts
        digit = self.digit
        upper = self.upper
        lower = self.lower
        special = self.special

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "org_id": org_id,
            }
        )
        if min_length is not UNSET:
            field_dict["min_length"] = min_length
        if expiration is not UNSET:
            field_dict["expiration"] = expiration
        if max_similarity is not UNSET:
            field_dict["max_similarity"] = max_similarity
        if max_login_attempts is not UNSET:
            field_dict["max_login_attempts"] = max_login_attempts
        if digit is not UNSET:
            field_dict["digit"] = digit
        if upper is not UNSET:
            field_dict["upper"] = upper
        if lower is not UNSET:
            field_dict["lower"] = lower
        if special is not UNSET:
            field_dict["special"] = special

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        org_id = d.pop("org_id")

        min_length = d.pop("min_length", UNSET)

        expiration = d.pop("expiration", UNSET)

        max_similarity = d.pop("max_similarity", UNSET)

        max_login_attempts = d.pop("max_login_attempts", UNSET)

        digit = d.pop("digit", UNSET)

        upper = d.pop("upper", UNSET)

        lower = d.pop("lower", UNSET)

        special = d.pop("special", UNSET)

        full_password_policy = cls(
            org_id=org_id,
            min_length=min_length,
            expiration=expiration,
            max_similarity=max_similarity,
            max_login_attempts=max_login_attempts,
            digit=digit,
            upper=upper,
            lower=lower,
            special=special,
        )

        full_password_policy.additional_properties = d
        return full_password_policy

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.http_validation_error import HTTPValidationError
from ...models.indexing_task_out_expanded import IndexingTaskOutExpanded
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    task_id: int,
    details: Union[Unset, bool] = False,
) -> Dict[str, Any]:
    url = "{}/api/v1/tasks/indexing/{task_id}/".format(client.base_url, task_id=task_id)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "details": details,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[IndexingTaskOutExpanded, None, None, None, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = IndexingTaskOutExpanded.from_dict(response.json())

        return response_200
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 404:
        response_404 = None

        return response_404
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[IndexingTaskOutExpanded, None, None, None, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    task_id: int,
    details: Union[Unset, bool] = False,
) -> Response[Union[IndexingTaskOutExpanded, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        task_id=task_id,
        details=details,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    task_id: int,
    details: Union[Unset, bool] = False,
) -> Optional[Union[IndexingTaskOutExpanded, None, None, None, HTTPValidationError]]:
    """Get an indexing report.

    Users can only access the indexing tasks they ran.
    If the details parameter is set to true, the route will return details about the
    task (i.e., average size and document types), otherwise the route only returns
    basic information."""

    return sync_detailed(
        client=client,
        task_id=task_id,
        details=details,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    task_id: int,
    details: Union[Unset, bool] = False,
) -> Response[Union[IndexingTaskOutExpanded, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        task_id=task_id,
        details=details,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    task_id: int,
    details: Union[Unset, bool] = False,
) -> Optional[Union[IndexingTaskOutExpanded, None, None, None, HTTPValidationError]]:
    """Get an indexing report.

    Users can only access the indexing tasks they ran.
    If the details parameter is set to true, the route will return details about the
    task (i.e., average size and document types), otherwise the route only returns
    basic information."""

    return (
        await asyncio_detailed(
            client=client,
            task_id=task_id,
            details=details,
        )
    ).parsed



from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    metadata_import_id: int,
    validate: bool,
) -> Dict[str, Any]:
    url = "{}/api/v1/metadata/import/{metadata_import_id}/validate/".format(
        client.base_url, metadata_import_id=metadata_import_id
    )

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {
        "validate": validate,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[None, None, None, None, HTTPValidationError]]:
    if response.status_code == 202:
        response_202 = None

        return response_202
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 404:
        response_404 = None

        return response_404
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[None, None, None, None, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    metadata_import_id: int,
    validate: bool,
) -> Response[Union[None, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        metadata_import_id=metadata_import_id,
        validate=validate,
    )

    response = httpx.post(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    metadata_import_id: int,
    validate: bool,
) -> Optional[Union[None, None, None, None, HTTPValidationError]]:
    """Validates or cancel a checked metadata import task.

    If validated, the update task is set up and executed.

    Only orgadmins can validate or cancel metadata import tasks."""

    return sync_detailed(
        client=client,
        metadata_import_id=metadata_import_id,
        validate=validate,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    metadata_import_id: int,
    validate: bool,
) -> Response[Union[None, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        metadata_import_id=metadata_import_id,
        validate=validate,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    metadata_import_id: int,
    validate: bool,
) -> Optional[Union[None, None, None, None, HTTPValidationError]]:
    """Validates or cancel a checked metadata import task.

    If validated, the update task is set up and executed.

    Only orgadmins can validate or cancel metadata import tasks."""

    return (
        await asyncio_detailed(
            client=client,
            metadata_import_id=metadata_import_id,
            validate=validate,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NotificationCount")


@attr.s(auto_attribs=True)
class NotificationCount:
    """ Notification schema for notification count. """

    n_unseen: int
    n_seen: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        n_unseen = self.n_unseen
        n_seen = self.n_seen

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "n_unseen": n_unseen,
            }
        )
        if n_seen is not UNSET:
            field_dict["n_seen"] = n_seen

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        n_unseen = d.pop("n_unseen")

        n_seen = d.pop("n_seen", UNSET)

        notification_count = cls(
            n_unseen=n_unseen,
            n_seen=n_seen,
        )

        notification_count.additional_properties = d
        return notification_count

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.indexing_error_in_db_args import IndexingErrorInDBArgs

T = TypeVar("T", bound="IndexingErrorInDB")


@attr.s(auto_attribs=True)
class IndexingErrorInDB:
    """ Indexing error schema for database input. """

    file_path: str
    code: int
    message: str
    args: IndexingErrorInDBArgs
    task_id: int
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        file_path = self.file_path
        code = self.code
        message = self.message
        args = self.args.to_dict()

        task_id = self.task_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "file_path": file_path,
                "code": code,
                "message": message,
                "args": args,
                "task_id": task_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        file_path = d.pop("file_path")

        code = d.pop("code")

        message = d.pop("message")

        args = IndexingErrorInDBArgs.from_dict(d.pop("args"))

        task_id = d.pop("task_id")

        indexing_error_in_db = cls(
            file_path=file_path,
            code=code,
            message=message,
            args=args,
            task_id=task_id,
        )

        indexing_error_in_db.additional_properties = d
        return indexing_error_in_db

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.organization_out import OrganizationOut
from ...types import Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
) -> Dict[str, Any]:
    url = "{}/api/v1/organizations/".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[List[OrganizationOut], None]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = OrganizationOut.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 401:
        response_401 = None

        return response_401
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[List[OrganizationOut], None]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
) -> Response[Union[List[OrganizationOut], None]]:
    kwargs = _get_kwargs(
        client=client,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
) -> Optional[Union[List[OrganizationOut], None]]:
    """Gets all organizations the user has access to.

    If the user is sysadmin, it returns all of the system organizations.

    If they are from another role, they can only access their own organization."""

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
) -> Response[Union[List[OrganizationOut], None]]:
    kwargs = _get_kwargs(
        client=client,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
) -> Optional[Union[List[OrganizationOut], None]]:
    """Gets all organizations the user has access to.

    If the user is sysadmin, it returns all of the system organizations.

    If they are from another role, they can only access their own organization."""

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="TwoFactorRequirement")


@attr.s(auto_attribs=True)
class TwoFactorRequirement:
    """ Two factor autentication requirement schema. """

    user_id: int
    required: bool
    provisioning_uri: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user_id = self.user_id
        required = self.required
        provisioning_uri = self.provisioning_uri

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user_id": user_id,
                "required": required,
            }
        )
        if provisioning_uri is not UNSET:
            field_dict["provisioning_uri"] = provisioning_uri

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user_id = d.pop("user_id")

        required = d.pop("required")

        provisioning_uri = d.pop("provisioning_uri", UNSET)

        two_factor_requirement = cls(
            user_id=user_id,
            required=required,
            provisioning_uri=provisioning_uri,
        )

        two_factor_requirement.additional_properties = d
        return two_factor_requirement

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.background_task_out import BackgroundTaskOut
from ..models.doc_type import DocType
from ..models.indexing_error_in_db import IndexingErrorInDB
from ..types import UNSET, Unset

T = TypeVar("T", bound="IndexingTaskOutExpanded")


@attr.s(auto_attribs=True)
class IndexingTaskOutExpanded:
    """ Output indexing task schema from GET methods. """

    bkg_task_id: int
    org_id: int
    run_by: int
    report_id: int
    id: int
    started_on: datetime.datetime
    background_task: BackgroundTaskOut
    errors: List[IndexingErrorInDB]
    finished_on: Union[Unset, datetime.datetime] = UNSET
    average_size: Union[Unset, int] = UNSET
    doc_types: Union[Unset, List[DocType]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bkg_task_id = self.bkg_task_id
        org_id = self.org_id
        run_by = self.run_by
        report_id = self.report_id
        id = self.id
        started_on = self.started_on.isoformat()

        background_task = self.background_task.to_dict()

        errors = []
        for errors_item_data in self.errors:
            errors_item = errors_item_data.to_dict()

            errors.append(errors_item)

        finished_on: Union[Unset, str] = UNSET
        if not isinstance(self.finished_on, Unset):
            finished_on = self.finished_on.isoformat()

        average_size = self.average_size
        doc_types: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.doc_types, Unset):
            doc_types = []
            for doc_types_item_data in self.doc_types:
                doc_types_item = doc_types_item_data.to_dict()

                doc_types.append(doc_types_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "bkg_task_id": bkg_task_id,
                "org_id": org_id,
                "run_by": run_by,
                "report_id": report_id,
                "id": id,
                "started_on": started_on,
                "background_task": background_task,
                "errors": errors,
            }
        )
        if finished_on is not UNSET:
            field_dict["finished_on"] = finished_on
        if average_size is not UNSET:
            field_dict["average_size"] = average_size
        if doc_types is not UNSET:
            field_dict["doc_types"] = doc_types

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bkg_task_id = d.pop("bkg_task_id")

        org_id = d.pop("org_id")

        run_by = d.pop("run_by")

        report_id = d.pop("report_id")

        id = d.pop("id")

        started_on = isoparse(d.pop("started_on"))

        background_task = BackgroundTaskOut.from_dict(d.pop("background_task"))

        errors = []
        _errors = d.pop("errors")
        for errors_item_data in _errors:
            errors_item = IndexingErrorInDB.from_dict(errors_item_data)

            errors.append(errors_item)

        finished_on: Union[Unset, datetime.datetime] = UNSET
        _finished_on = d.pop("finished_on", UNSET)
        if not isinstance(_finished_on, Unset):
            finished_on = isoparse(_finished_on)

        average_size = d.pop("average_size", UNSET)

        doc_types = []
        _doc_types = d.pop("doc_types", UNSET)
        for doc_types_item_data in _doc_types or []:
            doc_types_item = DocType.from_dict(doc_types_item_data)

            doc_types.append(doc_types_item)

        indexing_task_out_expanded = cls(
            bkg_task_id=bkg_task_id,
            org_id=org_id,
            run_by=run_by,
            report_id=report_id,
            id=id,
            started_on=started_on,
            background_task=background_task,
            errors=errors,
            finished_on=finished_on,
            average_size=average_size,
            doc_types=doc_types,
        )

        indexing_task_out_expanded.additional_properties = d
        return indexing_task_out_expanded

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.chunk_result import ChunkResult
from ..models.search_result_chunks_chunks_item import SearchResultChunksChunksItem
from ..models.search_result_pipeline_logs import SearchResultPipelineLogs
from ..types import UNSET, Unset

T = TypeVar("T", bound="SearchResultChunks")


@attr.s(auto_attribs=True)
class SearchResultChunks:
    """ Schema for search result chunks. """

    query_id: int
    chunks: Union[List[ChunkResult], List[SearchResultChunksChunksItem]]
    pipeline_logs: SearchResultPipelineLogs
    stripped_query: List[str]
    suggested_query: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        query_id = self.query_id
        if isinstance(self.chunks, list):
            chunks = []
            for chunks_item_data in self.chunks:
                chunks_item = chunks_item_data.to_dict()

                chunks.append(chunks_item)

        else:
            chunks = []
            for chunks_item_data in self.chunks:
                chunks_item = chunks_item_data.to_dict()

                chunks.append(chunks_item)

        pipeline_logs = self.pipeline_logs.to_dict()

        stripped_query = self.stripped_query

        suggested_query = self.suggested_query

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "query_id": query_id,
                "chunks": chunks,
                "pipeline_logs": pipeline_logs,
                "stripped_query": stripped_query,
            }
        )
        if suggested_query is not UNSET:
            field_dict["suggested_query"] = suggested_query

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        query_id = d.pop("query_id")

        def _parse_chunks(data: Any) -> Union[List[ChunkResult], List[SearchResultChunksChunksItem]]:
            data = None if isinstance(data, Unset) else data
            chunks: Union[List[ChunkResult], List[SearchResultChunksChunksItem]]
            try:
                chunks = UNSET
                _chunks = data
                for chunks_item_data in _chunks:
                    chunks_item = ChunkResult.from_dict(chunks_item_data)

                    chunks.append(chunks_item)

                return chunks
            except:  # noqa: E722
                pass
            chunks = UNSET
            _chunks = data
            for chunks_item_data in _chunks:
                chunks_item = SearchResultChunksChunksItem.from_dict(chunks_item_data)

                chunks.append(chunks_item)

            return chunks

        chunks = _parse_chunks(d.pop("chunks"))

        pipeline_logs = SearchResultPipelineLogs.from_dict(d.pop("pipeline_logs"))

        stripped_query = cast(List[str], d.pop("stripped_query"))

        suggested_query = d.pop("suggested_query", UNSET)

        search_result_chunks = cls(
            query_id=query_id,
            chunks=chunks,
            pipeline_logs=pipeline_logs,
            stripped_query=stripped_query,
            suggested_query=suggested_query,
        )

        search_result_chunks.additional_properties = d
        return search_result_chunks

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="OrganizationAdminIn")


@attr.s(auto_attribs=True)
class OrganizationAdminIn:
    """ Organization schema to admin input for POST methods. """

    first_name: str
    last_name: str
    email: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        first_name = self.first_name
        last_name = self.last_name
        email = self.email

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "first_name": first_name,
                "last_name": last_name,
                "email": email,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        first_name = d.pop("first_name")

        last_name = d.pop("last_name")

        email = d.pop("email")

        organization_admin_in = cls(
            first_name=first_name,
            last_name=last_name,
            email=email,
        )

        organization_admin_in.additional_properties = d
        return organization_admin_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

from ..models.folder_details import FolderDetails

T = TypeVar("T", bound="GroupDetails")


@attr.s(auto_attribs=True)
class GroupDetails:
    """ Group schema to output from GET groups/details. """

    name: str
    id: int
    org_id: int
    created_on: datetime.datetime
    read_rights: List[FolderDetails]
    write_rights: List[FolderDetails]
    n_users: int
    n_docs: int
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        id = self.id
        org_id = self.org_id
        created_on = self.created_on.isoformat()

        read_rights = []
        for read_rights_item_data in self.read_rights:
            read_rights_item = read_rights_item_data.to_dict()

            read_rights.append(read_rights_item)

        write_rights = []
        for write_rights_item_data in self.write_rights:
            write_rights_item = write_rights_item_data.to_dict()

            write_rights.append(write_rights_item)

        n_users = self.n_users
        n_docs = self.n_docs

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "id": id,
                "org_id": org_id,
                "created_on": created_on,
                "read_rights": read_rights,
                "write_rights": write_rights,
                "n_users": n_users,
                "n_docs": n_docs,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        id = d.pop("id")

        org_id = d.pop("org_id")

        created_on = isoparse(d.pop("created_on"))

        read_rights = []
        _read_rights = d.pop("read_rights")
        for read_rights_item_data in _read_rights:
            read_rights_item = FolderDetails.from_dict(read_rights_item_data)

            read_rights.append(read_rights_item)

        write_rights = []
        _write_rights = d.pop("write_rights")
        for write_rights_item_data in _write_rights:
            write_rights_item = FolderDetails.from_dict(write_rights_item_data)

            write_rights.append(write_rights_item)

        n_users = d.pop("n_users")

        n_docs = d.pop("n_docs")

        group_details = cls(
            name=name,
            id=id,
            org_id=org_id,
            created_on=created_on,
            read_rights=read_rights,
            write_rights=write_rights,
            n_users=n_users,
            n_docs=n_docs,
        )

        group_details.additional_properties = d
        return group_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.search_result_pipeline_logs_borda_chunks import SearchResultPipelineLogsBordaChunks
from ..models.search_result_pipeline_logs_documents import SearchResultPipelineLogsDocuments
from ..models.search_result_pipeline_logs_lexical_chunks import SearchResultPipelineLogsLexicalChunks
from ..models.search_result_pipeline_logs_semantic_chunks import SearchResultPipelineLogsSemanticChunks
from ..types import UNSET, Unset

T = TypeVar("T", bound="SearchResultPipelineLogs")


@attr.s(auto_attribs=True)
class SearchResultPipelineLogs:
    """ Schema for search pipeline logs """

    documents: Union[SearchResultPipelineLogsDocuments, Unset] = UNSET
    lexical_chunks: Union[SearchResultPipelineLogsLexicalChunks, Unset] = UNSET
    semantic_chunks: Union[SearchResultPipelineLogsSemanticChunks, Unset] = UNSET
    borda_chunks: Union[SearchResultPipelineLogsBordaChunks, Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        documents: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.documents, Unset):
            documents = self.documents.to_dict()

        lexical_chunks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.lexical_chunks, Unset):
            lexical_chunks = self.lexical_chunks.to_dict()

        semantic_chunks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.semantic_chunks, Unset):
            semantic_chunks = self.semantic_chunks.to_dict()

        borda_chunks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.borda_chunks, Unset):
            borda_chunks = self.borda_chunks.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if documents is not UNSET:
            field_dict["documents"] = documents
        if lexical_chunks is not UNSET:
            field_dict["lexical_chunks"] = lexical_chunks
        if semantic_chunks is not UNSET:
            field_dict["semantic_chunks"] = semantic_chunks
        if borda_chunks is not UNSET:
            field_dict["borda_chunks"] = borda_chunks

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        documents: Union[SearchResultPipelineLogsDocuments, Unset] = UNSET
        _documents = d.pop("documents", UNSET)
        if not isinstance(_documents, Unset):
            documents = SearchResultPipelineLogsDocuments.from_dict(_documents)

        lexical_chunks: Union[SearchResultPipelineLogsLexicalChunks, Unset] = UNSET
        _lexical_chunks = d.pop("lexical_chunks", UNSET)
        if not isinstance(_lexical_chunks, Unset):
            lexical_chunks = SearchResultPipelineLogsLexicalChunks.from_dict(_lexical_chunks)

        semantic_chunks: Union[SearchResultPipelineLogsSemanticChunks, Unset] = UNSET
        _semantic_chunks = d.pop("semantic_chunks", UNSET)
        if not isinstance(_semantic_chunks, Unset):
            semantic_chunks = SearchResultPipelineLogsSemanticChunks.from_dict(_semantic_chunks)

        borda_chunks: Union[SearchResultPipelineLogsBordaChunks, Unset] = UNSET
        _borda_chunks = d.pop("borda_chunks", UNSET)
        if not isinstance(_borda_chunks, Unset):
            borda_chunks = SearchResultPipelineLogsBordaChunks.from_dict(_borda_chunks)

        search_result_pipeline_logs = cls(
            documents=documents,
            lexical_chunks=lexical_chunks,
            semantic_chunks=semantic_chunks,
            borda_chunks=borda_chunks,
        )

        search_result_pipeline_logs.additional_properties = d
        return search_result_pipeline_logs

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.task_status import TaskStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="MetadataImportUpdate")


@attr.s(auto_attribs=True)
class MetadataImportUpdate:
    """ Metadata import PUT schema. """

    status: TaskStatus
    n_found_docs: Union[Unset, int] = UNSET
    n_notfound_docs: Union[Unset, int] = UNSET
    n_created_metadata: Union[Unset, int] = UNSET
    n_replaced_metadata: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status = self.status.value

        n_found_docs = self.n_found_docs
        n_notfound_docs = self.n_notfound_docs
        n_created_metadata = self.n_created_metadata
        n_replaced_metadata = self.n_replaced_metadata

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
            }
        )
        if n_found_docs is not UNSET:
            field_dict["n_found_docs"] = n_found_docs
        if n_notfound_docs is not UNSET:
            field_dict["n_notfound_docs"] = n_notfound_docs
        if n_created_metadata is not UNSET:
            field_dict["n_created_metadata"] = n_created_metadata
        if n_replaced_metadata is not UNSET:
            field_dict["n_replaced_metadata"] = n_replaced_metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = TaskStatus(d.pop("status"))

        n_found_docs = d.pop("n_found_docs", UNSET)

        n_notfound_docs = d.pop("n_notfound_docs", UNSET)

        n_created_metadata = d.pop("n_created_metadata", UNSET)

        n_replaced_metadata = d.pop("n_replaced_metadata", UNSET)

        metadata_import_update = cls(
            status=status,
            n_found_docs=n_found_docs,
            n_notfound_docs=n_notfound_docs,
            n_created_metadata=n_created_metadata,
            n_replaced_metadata=n_replaced_metadata,
        )

        metadata_import_update.additional_properties = d
        return metadata_import_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.extract_paragraph_config import ExtractParagraphConfig
from ..models.extract_question_config import ExtractQuestionConfig
from ..models.extract_type import ExtractType
from ..models.extract_value_config import ExtractValueConfig
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExtractModelOut")


@attr.s(auto_attribs=True)
class ExtractModelOut:
    """ Extract model schema to be received on the GET /extract/models response. """

    name: str
    type: ExtractType
    config: Union[ExtractValueConfig, ExtractParagraphConfig, ExtractQuestionConfig]
    org_id: int
    id: int
    created_on: datetime.datetime
    questionnaire_model_id: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        type = self.type.value

        if isinstance(self.config, ExtractValueConfig):
            config = self.config.to_dict()

        elif isinstance(self.config, ExtractParagraphConfig):
            config = self.config.to_dict()

        else:
            config = self.config.to_dict()

        org_id = self.org_id
        id = self.id
        created_on = self.created_on.isoformat()

        questionnaire_model_id = self.questionnaire_model_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "type": type,
                "config": config,
                "org_id": org_id,
                "id": id,
                "created_on": created_on,
            }
        )
        if questionnaire_model_id is not UNSET:
            field_dict["questionnaire_model_id"] = questionnaire_model_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        type = ExtractType(d.pop("type"))

        def _parse_config(data: Any) -> Union[ExtractValueConfig, ExtractParagraphConfig, ExtractQuestionConfig]:
            data = None if isinstance(data, Unset) else data
            config: Union[ExtractValueConfig, ExtractParagraphConfig, ExtractQuestionConfig]
            try:
                config = ExtractValueConfig.from_dict(data)

                return config
            except:  # noqa: E722
                pass
            try:
                config = ExtractParagraphConfig.from_dict(data)

                return config
            except:  # noqa: E722
                pass
            config = ExtractQuestionConfig.from_dict(data)

            return config

        config = _parse_config(d.pop("config"))

        org_id = d.pop("org_id")

        id = d.pop("id")

        created_on = isoparse(d.pop("created_on"))

        questionnaire_model_id = d.pop("questionnaire_model_id", UNSET)

        extract_model_out = cls(
            name=name,
            type=type,
            config=config,
            org_id=org_id,
            id=id,
            created_on=created_on,
            questionnaire_model_id=questionnaire_model_id,
        )

        extract_model_out.additional_properties = d
        return extract_model_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.timezone import Timezone
from ..models.user_language import UserLanguage
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserAccountUpdate")


@attr.s(auto_attribs=True)
class UserAccountUpdate:
    """ User schema to receive from PUT /account endpoint. """

    email: Union[Unset, str] = UNSET
    first_name: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    timezone: Union[Unset, Timezone] = UNSET
    language: Union[Unset, UserLanguage] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        email = self.email
        first_name = self.first_name
        last_name = self.last_name
        timezone: Union[Unset, Timezone] = UNSET
        if not isinstance(self.timezone, Unset):
            timezone = self.timezone

        language: Union[Unset, UserLanguage] = UNSET
        if not isinstance(self.language, Unset):
            language = self.language

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if email is not UNSET:
            field_dict["email"] = email
        if first_name is not UNSET:
            field_dict["first_name"] = first_name
        if last_name is not UNSET:
            field_dict["last_name"] = last_name
        if timezone is not UNSET:
            field_dict["timezone"] = timezone
        if language is not UNSET:
            field_dict["language"] = language

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        email = d.pop("email", UNSET)

        first_name = d.pop("first_name", UNSET)

        last_name = d.pop("last_name", UNSET)

        timezone: Union[Unset, Timezone] = UNSET
        _timezone = d.pop("timezone", UNSET)
        if not isinstance(_timezone, Unset):
            timezone = Timezone(_timezone)

        language: Union[Unset, UserLanguage] = UNSET
        _language = d.pop("language", UNSET)
        if not isinstance(_language, Unset):
            language = UserLanguage(_language)

        user_account_update = cls(
            email=email,
            first_name=first_name,
            last_name=last_name,
            timezone=timezone,
            language=language,
        )

        user_account_update.additional_properties = d
        return user_account_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.chunk_bookmark_get_coordinates import ChunkBookmarkGetCoordinates
from ..models.chunk_bookmark_get_div_idxs import ChunkBookmarkGetDivIdxs
from ..models.chunk_bookmark_get_metadata import ChunkBookmarkGetMetadata
from ..models.chunk_bookmark_get_query import ChunkBookmarkGetQuery
from ..types import UNSET, Unset

T = TypeVar("T", bound="ChunkBookmarkGet")


@attr.s(auto_attribs=True)
class ChunkBookmarkGet:
    """ Chunk bookmark schema for the get by id endpoint. """

    collection_id: int
    id: int
    query: ChunkBookmarkGetQuery
    version_id: int
    chunk_id: str
    chunk_text: str
    created_on: datetime.datetime
    from_semantic: bool
    metadata: ChunkBookmarkGetMetadata
    div_idxs: ChunkBookmarkGetDivIdxs
    pages: List[int]
    display_name: str
    coordinates: Union[ChunkBookmarkGetCoordinates, Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        collection_id = self.collection_id
        id = self.id
        query = self.query.to_dict()

        version_id = self.version_id
        chunk_id = self.chunk_id
        chunk_text = self.chunk_text
        created_on = self.created_on.isoformat()

        from_semantic = self.from_semantic
        metadata = self.metadata.to_dict()

        div_idxs = self.div_idxs.to_dict()

        pages = self.pages

        display_name = self.display_name
        coordinates: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.coordinates, Unset):
            coordinates = self.coordinates.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "collection_id": collection_id,
                "id": id,
                "query": query,
                "version_id": version_id,
                "chunk_id": chunk_id,
                "chunk_text": chunk_text,
                "created_on": created_on,
                "from_semantic": from_semantic,
                "metadata": metadata,
                "div_idxs": div_idxs,
                "pages": pages,
                "display_name": display_name,
            }
        )
        if coordinates is not UNSET:
            field_dict["coordinates"] = coordinates

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        collection_id = d.pop("collection_id")

        id = d.pop("id")

        query = ChunkBookmarkGetQuery.from_dict(d.pop("query"))

        version_id = d.pop("version_id")

        chunk_id = d.pop("chunk_id")

        chunk_text = d.pop("chunk_text")

        created_on = isoparse(d.pop("created_on"))

        from_semantic = d.pop("from_semantic")

        metadata = ChunkBookmarkGetMetadata.from_dict(d.pop("metadata"))

        div_idxs = ChunkBookmarkGetDivIdxs.from_dict(d.pop("div_idxs"))

        pages = cast(List[int], d.pop("pages"))

        display_name = d.pop("display_name")

        coordinates: Union[ChunkBookmarkGetCoordinates, Unset] = UNSET
        _coordinates = d.pop("coordinates", UNSET)
        if not isinstance(_coordinates, Unset):
            coordinates = ChunkBookmarkGetCoordinates.from_dict(_coordinates)

        chunk_bookmark_get = cls(
            collection_id=collection_id,
            id=id,
            query=query,
            version_id=version_id,
            chunk_id=chunk_id,
            chunk_text=chunk_text,
            created_on=created_on,
            from_semantic=from_semantic,
            metadata=metadata,
            div_idxs=div_idxs,
            pages=pages,
            display_name=display_name,
            coordinates=coordinates,
        )

        chunk_bookmark_get.additional_properties = d
        return chunk_bookmark_get

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.http_validation_error import HTTPValidationError
from ...models.indexing_task_out import IndexingTaskOut
from ...models.indexing_task_update import IndexingTaskUpdate
from ...types import Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    task_id: int,
    json_body: IndexingTaskUpdate,
) -> Dict[str, Any]:
    url = "{}/api/v1/tasks/indexing/{task_id}/update/".format(client.base_url, task_id=task_id)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[IndexingTaskOut, None, None, None, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = IndexingTaskOut.from_dict(response.json())

        return response_200
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 404:
        response_404 = None

        return response_404
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[IndexingTaskOut, None, None, None, HTTPValidationError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    task_id: int,
    json_body: IndexingTaskUpdate,
) -> Response[Union[IndexingTaskOut, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        task_id=task_id,
        json_body=json_body,
    )

    response = httpx.put(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    task_id: int,
    json_body: IndexingTaskUpdate,
) -> Optional[Union[IndexingTaskOut, None, None, None, HTTPValidationError]]:
    """Update an indexing task by passing a number of processed items.
    This number is added to the number of already processed documents.
    The route automatically sets the task as completed,
    if the total number of documents to process is reached.

    Only services can update indexing reports."""

    return sync_detailed(
        client=client,
        task_id=task_id,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    task_id: int,
    json_body: IndexingTaskUpdate,
) -> Response[Union[IndexingTaskOut, None, None, None, HTTPValidationError]]:
    kwargs = _get_kwargs(
        client=client,
        task_id=task_id,
        json_body=json_body,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.put(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    task_id: int,
    json_body: IndexingTaskUpdate,
) -> Optional[Union[IndexingTaskOut, None, None, None, HTTPValidationError]]:
    """Update an indexing task by passing a number of processed items.
    This number is added to the number of already processed documents.
    The route automatically sets the task as completed,
    if the total number of documents to process is reached.

    Only services can update indexing reports."""

    return (
        await asyncio_detailed(
            client=client,
            task_id=task_id,
            json_body=json_body,
        )
    ).parsed



import datetime
from io import BytesIO
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.chunk_result_coordinates import ChunkResultCoordinates
from ..models.chunk_result_div_idxs import ChunkResultDivIdxs
from ..models.chunk_result_duplicates_item import ChunkResultDuplicatesItem
from ..models.chunk_result_metadata import ChunkResultMetadata
from ..types import UNSET, File, Unset

T = TypeVar("T", bound="ChunkResult")


@attr.s(auto_attribs=True)
class ChunkResult:
    """ Chunk result schema. """

    name: str
    display_name: str
    folder_id: int
    item_id: int
    extension: str
    is_current_version: bool
    size: int
    ocr: bool
    id: str
    version_id: int
    div_idxs: ChunkResultDivIdxs
    pages: List[int]
    title: Union[Unset, str] = UNSET
    author: Union[Unset, str] = UNSET
    metadata: Union[ChunkResultMetadata, Unset] = UNSET
    num_pages: Union[Unset, int] = UNSET
    created_on: Union[Unset, datetime.datetime] = UNSET
    updated_on: Union[Unset, datetime.datetime] = UNSET
    text: Union[Unset, str] = UNSET
    image: Union[Unset, File] = UNSET
    coordinates: Union[ChunkResultCoordinates, Unset] = UNSET
    is_semantic: Union[Unset, bool] = False
    duplicates: Union[Unset, List[ChunkResultDuplicatesItem]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        display_name = self.display_name
        folder_id = self.folder_id
        item_id = self.item_id
        extension = self.extension
        is_current_version = self.is_current_version
        size = self.size
        ocr = self.ocr
        id = self.id
        version_id = self.version_id
        div_idxs = self.div_idxs.to_dict()

        pages = self.pages

        title = self.title
        author = self.author
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        num_pages = self.num_pages
        created_on: Union[Unset, str] = UNSET
        if not isinstance(self.created_on, Unset):
            created_on = self.created_on.isoformat()

        updated_on: Union[Unset, str] = UNSET
        if not isinstance(self.updated_on, Unset):
            updated_on = self.updated_on.isoformat()

        text = self.text
        image: Union[Unset, File] = UNSET
        if not isinstance(self.image, Unset):
            image = self.image.to_tuple()

        coordinates: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.coordinates, Unset):
            coordinates = self.coordinates.to_dict()

        is_semantic = self.is_semantic
        duplicates: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.duplicates, Unset):
            duplicates = []
            for duplicates_item_data in self.duplicates:
                duplicates_item = duplicates_item_data.to_dict()

                duplicates.append(duplicates_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "display_name": display_name,
                "folder_id": folder_id,
                "item_id": item_id,
                "extension": extension,
                "is_current_version": is_current_version,
                "size": size,
                "ocr": ocr,
                "id": id,
                "version_id": version_id,
                "div_idxs": div_idxs,
                "pages": pages,
            }
        )
        if title is not UNSET:
            field_dict["title"] = title
        if author is not UNSET:
            field_dict["author"] = author
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if num_pages is not UNSET:
            field_dict["num_pages"] = num_pages
        if created_on is not UNSET:
            field_dict["created_on"] = created_on
        if updated_on is not UNSET:
            field_dict["updated_on"] = updated_on
        if text is not UNSET:
            field_dict["text"] = text
        if image is not UNSET:
            field_dict["image"] = image
        if coordinates is not UNSET:
            field_dict["coordinates"] = coordinates
        if is_semantic is not UNSET:
            field_dict["is_semantic"] = is_semantic
        if duplicates is not UNSET:
            field_dict["duplicates"] = duplicates

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        display_name = d.pop("display_name")

        folder_id = d.pop("folder_id")

        item_id = d.pop("item_id")

        extension = d.pop("extension")

        is_current_version = d.pop("is_current_version")

        size = d.pop("size")

        ocr = d.pop("ocr")

        id = d.pop("id")

        version_id = d.pop("version_id")

        div_idxs = ChunkResultDivIdxs.from_dict(d.pop("div_idxs"))

        pages = cast(List[int], d.pop("pages"))

        title = d.pop("title", UNSET)

        author = d.pop("author", UNSET)

        metadata: Union[ChunkResultMetadata, Unset] = UNSET
        _metadata = d.pop("metadata", UNSET)
        if not isinstance(_metadata, Unset):
            metadata = ChunkResultMetadata.from_dict(_metadata)

        num_pages = d.pop("num_pages", UNSET)

        created_on: Union[Unset, datetime.datetime] = UNSET
        _created_on = d.pop("created_on", UNSET)
        if not isinstance(_created_on, Unset):
            created_on = isoparse(_created_on)

        updated_on: Union[Unset, datetime.datetime] = UNSET
        _updated_on = d.pop("updated_on", UNSET)
        if not isinstance(_updated_on, Unset):
            updated_on = isoparse(_updated_on)

        text = d.pop("text", UNSET)

        image: Union[Unset, File] = UNSET
        _image = d.pop("image", UNSET)
        if not isinstance(_image, Unset):
            image = File(payload=BytesIO(_image))

        coordinates: Union[ChunkResultCoordinates, Unset] = UNSET
        _coordinates = d.pop("coordinates", UNSET)
        if not isinstance(_coordinates, Unset):
            coordinates = ChunkResultCoordinates.from_dict(_coordinates)

        is_semantic = d.pop("is_semantic", UNSET)

        duplicates = []
        _duplicates = d.pop("duplicates", UNSET)
        for duplicates_item_data in _duplicates or []:
            duplicates_item = ChunkResultDuplicatesItem.from_dict(duplicates_item_data)

            duplicates.append(duplicates_item)

        chunk_result = cls(
            name=name,
            display_name=display_name,
            folder_id=folder_id,
            item_id=item_id,
            extension=extension,
            is_current_version=is_current_version,
            size=size,
            ocr=ocr,
            id=id,
            version_id=version_id,
            div_idxs=div_idxs,
            pages=pages,
            title=title,
            author=author,
            metadata=metadata,
            num_pages=num_pages,
            created_on=created_on,
            updated_on=updated_on,
            text=text,
            image=image,
            coordinates=coordinates,
            is_semantic=is_semantic,
            duplicates=duplicates,
        )

        chunk_result.additional_properties = d
        return chunk_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.questionnaire_out_source_query import QuestionnaireOutSourceQuery
from ..models.questionnaire_status import QuestionnaireStatus
from ..models.source_filters import SourceFilters
from ..models.source_type import SourceType
from ..types import UNSET, Unset

T = TypeVar("T", bound="QuestionnaireOut")


@attr.s(auto_attribs=True)
class QuestionnaireOut:
    """ Questionnaire schema for API return. """

    model_id: int
    org_id: int
    name: str
    created_by_id: int
    created_by_name: str
    source_type: SourceType
    source_filters: SourceFilters
    source_query: QuestionnaireOutSourceQuery
    id: int
    status: QuestionnaireStatus
    created_on: datetime.datetime
    total_documents: Union[Unset, int] = 0
    per_folder: Union[Unset, bool] = False
    completed_on: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        model_id = self.model_id
        org_id = self.org_id
        name = self.name
        created_by_id = self.created_by_id
        created_by_name = self.created_by_name
        source_type = self.source_type.value

        source_filters = self.source_filters.to_dict()

        source_query = self.source_query.to_dict()

        id = self.id
        status = self.status.value

        created_on = self.created_on.isoformat()

        total_documents = self.total_documents
        per_folder = self.per_folder
        completed_on: Union[Unset, str] = UNSET
        if not isinstance(self.completed_on, Unset):
            completed_on = self.completed_on.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "model_id": model_id,
                "org_id": org_id,
                "name": name,
                "created_by_id": created_by_id,
                "created_by_name": created_by_name,
                "source_type": source_type,
                "source_filters": source_filters,
                "source_query": source_query,
                "id": id,
                "status": status,
                "created_on": created_on,
            }
        )
        if total_documents is not UNSET:
            field_dict["total_documents"] = total_documents
        if per_folder is not UNSET:
            field_dict["per_folder"] = per_folder
        if completed_on is not UNSET:
            field_dict["completed_on"] = completed_on

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        model_id = d.pop("model_id")

        org_id = d.pop("org_id")

        name = d.pop("name")

        created_by_id = d.pop("created_by_id")

        created_by_name = d.pop("created_by_name")

        source_type = SourceType(d.pop("source_type"))

        source_filters = SourceFilters.from_dict(d.pop("source_filters"))

        source_query = QuestionnaireOutSourceQuery.from_dict(d.pop("source_query"))

        id = d.pop("id")

        status = QuestionnaireStatus(d.pop("status"))

        created_on = isoparse(d.pop("created_on"))

        total_documents = d.pop("total_documents", UNSET)

        per_folder = d.pop("per_folder", UNSET)

        completed_on: Union[Unset, datetime.datetime] = UNSET
        _completed_on = d.pop("completed_on", UNSET)
        if not isinstance(_completed_on, Unset):
            completed_on = isoparse(_completed_on)

        questionnaire_out = cls(
            model_id=model_id,
            org_id=org_id,
            name=name,
            created_by_id=created_by_id,
            created_by_name=created_by_name,
            source_type=source_type,
            source_filters=source_filters,
            source_query=source_query,
            id=id,
            status=status,
            created_on=created_on,
            total_documents=total_documents,
            per_folder=per_folder,
            completed_on=completed_on,
        )

        questionnaire_out.additional_properties = d
        return questionnaire_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.background_task_update_status_info import BackgroundTaskUpdateStatusInfo
from ..models.task_status import TaskStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="BackgroundTaskUpdateStatus")


@attr.s(auto_attribs=True)
class BackgroundTaskUpdateStatus:
    """Background task schema for updating
    a task status and number of items to be processed.

    An optional report id can be specified."""

    status: TaskStatus
    total_items: Union[Unset, int] = UNSET
    info: Union[BackgroundTaskUpdateStatusInfo, Unset] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status = self.status.value

        total_items = self.total_items
        info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.info, Unset):
            info = self.info.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
            }
        )
        if total_items is not UNSET:
            field_dict["total_items"] = total_items
        if info is not UNSET:
            field_dict["info"] = info

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = TaskStatus(d.pop("status"))

        total_items = d.pop("total_items", UNSET)

        info: Union[BackgroundTaskUpdateStatusInfo, Unset] = UNSET
        _info = d.pop("info", UNSET)
        if not isinstance(_info, Unset):
            info = BackgroundTaskUpdateStatusInfo.from_dict(_info)

        background_task_update_status = cls(
            status=status,
            total_items=total_items,
            info=info,
        )

        background_task_update_status.additional_properties = d
        return background_task_update_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.organization_admin_in import OrganizationAdminIn
from ..models.organization_create import OrganizationCreate

T = TypeVar("T", bound="OrganizationCreateIn")


@attr.s(auto_attribs=True)
class OrganizationCreateIn:
    """ Organization schema for POST input. """

    org: OrganizationCreate
    admin: OrganizationAdminIn
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        org = self.org.to_dict()

        admin = self.admin.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "org": org,
                "admin": admin,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        org = OrganizationCreate.from_dict(d.pop("org"))

        admin = OrganizationAdminIn.from_dict(d.pop("admin"))

        organization_create_in = cls(
            org=org,
            admin=admin,
        )

        organization_create_in.additional_properties = d
        return organization_create_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

from ..models.license_type import LicenseType
from ..models.product import Product

T = TypeVar("T", bound="OrganizationCreate")


@attr.s(auto_attribs=True)
class OrganizationCreate:
    """ Organization schema to receive from POST method. """

    name: str
    product: Product
    license_expires_on: datetime.datetime
    license_type: LicenseType
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        product = self.product.value

        license_expires_on = self.license_expires_on.isoformat()

        license_type = self.license_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "product": product,
                "license_expires_on": license_expires_on,
                "license_type": license_type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        product = Product(d.pop("product"))

        license_expires_on = isoparse(d.pop("license_expires_on"))

        license_type = LicenseType(d.pop("license_type"))

        organization_create = cls(
            name=name,
            product=product,
            license_expires_on=license_expires_on,
            license_type=license_type,
        )

        organization_create.additional_properties = d
        return organization_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .access_token import AccessToken
from .analyzer_language import AnalyzerLanguage
from .archive_type import ArchiveType
from .archives_out import ArchivesOut
from .background_task_in import BackgroundTaskIn
from .background_task_out import BackgroundTaskOut
from .background_task_update_items import BackgroundTaskUpdateItems
from .background_task_update_status import BackgroundTaskUpdateStatus
from .background_task_update_status_info import BackgroundTaskUpdateStatusInfo
from .block_from_digester import BlockFromDigester
from .body_create_file_api_v1_files_post import BodyCreateFileApiV1Files_Post
from .body_post_file_version_api_v1_files_versions_item_item_id__post import (
    BodyPostFileVersionApiV1FilesVersionsItem_ItemId__Post,
)
from .body_post_metadata_import_api_v1_metadata_import_post import BodyPostMetadataImportApiV1MetadataImport_Post
from .body_post_questionnaire_run_api_v1_extract_questionnaire_start_post import (
    BodyPostQuestionnaireRunApiV1ExtractQuestionnaireStart_Post,
)
from .body_post_refresh_token_api_v1_refresh_token_post import BodyPostRefreshTokenApiV1RefreshToken_Post
from .body_post_user_login_api_v1_login_post import BodyPostUserLoginApiV1Login_Post
from .body_put_metadata_display_api_v1_metadata_display_put import BodyPutMetadataDisplayApiV1MetadataDisplay_Put
from .body_put_synonyms_api_v1_search_config_synonyms_put import BodyPutSynonymsApiV1SearchConfigSynonyms_Put
from .body_put_user_account_api_v1_users_import_post import BodyPutUserAccountApiV1UsersImport_Post
from .chunk_bookmark_create import ChunkBookmarkCreate
from .chunk_bookmark_get import ChunkBookmarkGet
from .chunk_bookmark_get_coordinates import ChunkBookmarkGetCoordinates
from .chunk_bookmark_get_div_idxs import ChunkBookmarkGetDivIdxs
from .chunk_bookmark_get_metadata import ChunkBookmarkGetMetadata
from .chunk_bookmark_get_query import ChunkBookmarkGetQuery
from .chunk_bookmark_in import ChunkBookmarkIn
from .chunk_bookmark_in_coordinates import ChunkBookmarkInCoordinates
from .chunk_bookmark_in_query import ChunkBookmarkInQuery
from .chunk_bookmark_out import ChunkBookmarkOut
from .chunk_bookmark_out_coordinates import ChunkBookmarkOutCoordinates
from .chunk_bookmark_out_div_idxs import ChunkBookmarkOutDivIdxs
from .chunk_bookmark_out_metadata import ChunkBookmarkOutMetadata
from .chunk_bookmark_out_query import ChunkBookmarkOutQuery
from .chunk_bookmark_result import ChunkBookmarkResult
from .chunk_collection_create import ChunkCollectionCreate
from .chunk_collection_out import ChunkCollectionOut
from .chunk_collection_update import ChunkCollectionUpdate
from .chunk_out import ChunkOut
from .chunk_out_coordinates import ChunkOutCoordinates
from .chunk_out_div_idxs import ChunkOutDivIdxs
from .chunk_out_metadata import ChunkOutMetadata
from .chunk_result import ChunkResult
from .chunk_result_coordinates import ChunkResultCoordinates
from .chunk_result_div_idxs import ChunkResultDivIdxs
from .chunk_result_duplicates_item import ChunkResultDuplicatesItem
from .chunk_result_metadata import ChunkResultMetadata
from .collection_create import CollectionCreate
from .collection_output import CollectionOutput
from .collection_update import CollectionUpdate
from .content_content_out import ContentContentOut
from .content_details_out import ContentDetailsOut
from .create_document_outline_json_body_item import CreateDocumentOutlineJsonBodyItem
from .display_policy import DisplayPolicy
from .display_policy_update import DisplayPolicyUpdate
from .doc_count import DocCount
from .doc_search_metadata import DocSearchMetadata
from .doc_search_metadata_name import DocSearchMetadataName
from .doc_search_out import DocSearchOut
from .doc_source import DocSource
from .doc_type import DocType
from .document_bookmark_create import DocumentBookmarkCreate
from .document_bookmark_current_version import DocumentBookmarkCurrentVersion
from .document_bookmark_output import DocumentBookmarkOutput
from .document_bookmark_update import DocumentBookmarkUpdate
from .document_outline_out import DocumentOutlineOut
from .document_outline_out_outline_item import DocumentOutlineOutOutlineItem
from .document_result import DocumentResult
from .document_result_metadata import DocumentResultMetadata
from .email_configuration_out import EmailConfigurationOut
from .email_configuration_update import EmailConfigurationUpdate
from .email_template_out import EmailTemplateOut
from .email_template_update import EmailTemplateUpdate
from .entity_in import EntityIn
from .entity_in_db import EntityInDB
from .event_out import EventOut
from .event_out_args import EventOutArgs
from .extract_config_out import ExtractConfigOut
from .extract_config_update import ExtractConfigUpdate
from .extract_model_create import ExtractModelCreate
from .extract_model_out import ExtractModelOut
from .extract_paragraph_config import ExtractParagraphConfig
from .extract_prediction_in import ExtractPredictionIn
from .extract_question_config import ExtractQuestionConfig
from .extract_result_out import ExtractResultOut
from .extract_result_out_metadata import ExtractResultOutMetadata
from .extract_result_out_version_metadata import ExtractResultOutVersionMetadata
from .extract_result_validate_in import ExtractResultValidateIn
from .extract_result_validate_out import ExtractResultValidateOut
from .extract_result_validate_out_metadata import ExtractResultValidateOutMetadata
from .extract_result_validate_out_version_metadata import ExtractResultValidateOutVersionMetadata
from .extract_task_create import ExtractTaskCreate
from .extract_task_out import ExtractTaskOut
from .extract_task_status import ExtractTaskStatus
from .extract_task_status_in import ExtractTaskStatusIn
from .extract_top_chunk import ExtractTopChunk
from .extract_top_chunk_out import ExtractTopChunkOut
from .extract_top_chunk_validate_out import ExtractTopChunkValidateOut
from .extract_top_chunk_validate_out_coordinates import ExtractTopChunkValidateOutCoordinates
from .extract_top_chunk_validate_out_div_idxs import ExtractTopChunkValidateOutDivIdxs
from .extract_type import ExtractType
from .extract_value_config import ExtractValueConfig
from .file_item_move import FileItemMove
from .file_item_out import FileItemOut
from .file_item_out_with_latest_version import FileItemOutWithLatestVersion
from .file_item_out_with_versions import FileItemOutWithVersions
from .file_item_rename import FileItemRename
from .filter_operator import FilterOperator
from .folder_create import FolderCreate
from .folder_details import FolderDetails
from .folder_move import FolderMove
from .folder_out import FolderOut
from .folder_out_by_group import FolderOutByGroup
from .folder_right import FolderRight
from .folder_update import FolderUpdate
from .full_password_policy import FullPasswordPolicy
from .get_version_styled_html_pages_response_get_version_styled_html_pages_api_v1_files_versions_version_id_styled_html_get import (
    GetVersionStyledHtmlPagesResponseGetVersionStyledHtmlPagesApiV1FilesVersionsVersionIdStyledHtmlGet,
)
from .google_drive_entity import GoogleDriveEntity
from .google_drives_out import GoogleDrivesOut
from .google_folders_out import GoogleFoldersOut
from .group_create_in import GroupCreateIn
from .group_details import GroupDetails
from .group_folder_rights import GroupFolderRights
from .group_import_report import GroupImportReport
from .group_out import GroupOut
from .group_update_in import GroupUpdateIn
from .http_validation_error import HTTPValidationError
from .id import ID
from .import_report import ImportReport
from .indexing_error_in import IndexingErrorIn
from .indexing_error_in_args import IndexingErrorInArgs
from .indexing_error_in_db import IndexingErrorInDB
from .indexing_error_in_db_args import IndexingErrorInDBArgs
from .indexing_status_in import IndexingStatusIn
from .indexing_task_create import IndexingTaskCreate
from .indexing_task_out import IndexingTaskOut
from .indexing_task_out_expanded import IndexingTaskOutExpanded
from .indexing_task_update import IndexingTaskUpdate
from .keyword_operator import KeywordOperator
from .keywords_direction import KeywordsDirection
from .keywords_scope import KeywordsScope
from .license_type import LicenseType
from .login import Login
from .member import Member
from .metadata_create import MetadataCreate
from .metadata_date_range import MetadataDateRange
from .metadata_display_out import MetadataDisplayOut
from .metadata_id import MetadataId
from .metadata_import_cleanup import MetadataImportCleanup
from .metadata_import_create_out import MetadataImportCreateOut
from .metadata_import_json import MetadataImportJSON
from .metadata_import_json_data import MetadataImportJSONData
from .metadata_import_json_data_additional_property import MetadataImportJSONDataAdditionalProperty
from .metadata_import_out import MetadataImportOut
from .metadata_import_update import MetadataImportUpdate
from .metadata_imports_data import MetadataImportsData
from .metadata_name_update import MetadataNameUpdate
from .metadata_out import MetadataOut
from .metadata_source import MetadataSource
from .metadata_type import MetadataType
from .notification_count import NotificationCount
from .notification_out import NotificationOut
from .notification_out_args import NotificationOutArgs
from .notification_out_info import NotificationOutInfo
from .notification_put_in import NotificationPutIn
from .order_by_users import OrderByUsers
from .organization_admin_in import OrganizationAdminIn
from .organization_admin_out import OrganizationAdminOut
from .organization_create import OrganizationCreate
from .organization_create_in import OrganizationCreateIn
from .organization_create_out import OrganizationCreateOut
from .organization_out import OrganizationOut
from .organization_update import OrganizationUpdate
from .password import Password
from .password_policy_expected import PasswordPolicyExpected
from .password_policy_update import PasswordPolicyUpdate
from .password_update_login import PasswordUpdateLogin
from .permission import Permission
from .permission_update import PermissionUpdate
from .product import Product
from .questionnaire import Questionnaire
from .questionnaire_create import QuestionnaireCreate
from .questionnaire_model_create import QuestionnaireModelCreate
from .questionnaire_model_out import QuestionnaireModelOut
from .questionnaire_out import QuestionnaireOut
from .questionnaire_out_source_query import QuestionnaireOutSourceQuery
from .questionnaire_status import QuestionnaireStatus
from .report_create import ReportCreate
from .report_output import ReportOutput
from .rights_create import RightsCreate
from .rights_list_out import RightsListOut
from .rights_update import RightsUpdate
from .search_configuration_out import SearchConfigurationOut
from .search_configuration_update import SearchConfigurationUpdate
from .search_filter import SearchFilter
from .search_query_keywords import SearchQueryKeywords
from .search_query_keywords_filters import SearchQueryKeywordsFilters
from .search_query_out import SearchQueryOut
from .search_query_out_filters import SearchQueryOutFilters
from .search_query_string_query import SearchQueryStringQuery
from .search_query_string_query_filters import SearchQueryStringQueryFilters
from .search_result_chunks import SearchResultChunks
from .search_result_chunks_chunks_item import SearchResultChunksChunksItem
from .search_result_documents import SearchResultDocuments
from .search_result_documents_documents_item import SearchResultDocumentsDocumentsItem
from .search_result_pipeline_logs import SearchResultPipelineLogs
from .search_result_pipeline_logs_borda_chunks import SearchResultPipelineLogsBordaChunks
from .search_result_pipeline_logs_documents import SearchResultPipelineLogsDocuments
from .search_result_pipeline_logs_lexical_chunks import SearchResultPipelineLogsLexicalChunks
from .search_result_pipeline_logs_semantic_chunks import SearchResultPipelineLogsSemanticChunks
from .source_filters import SourceFilters
from .source_type import SourceType
from .task import Task
from .task_status import TaskStatus
from .timezone import Timezone
from .total_count import TotalCount
from .two_factor_authentication import TwoFactorAuthentication
from .two_factor_base_credentials import TwoFactorBaseCredentials
from .two_factor_requirement import TwoFactorRequirement
from .user_account_update import UserAccountUpdate
from .user_admin_update import UserAdminUpdate
from .user_create import UserCreate
from .user_identity import UserIdentity
from .user_language import UserLanguage
from .user_out import UserOut
from .user_password_update import UserPasswordUpdate
from .user_role import UserRole
from .user_status import UserStatus
from .validation_error import ValidationError
from .value_type import ValueType
from .version import Version
from .version_metadata import VersionMetadata
from .versioning_policy import VersioningPolicy
from .versioning_policy_update import VersioningPolicyUpdate



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ReportCreate")


@attr.s(auto_attribs=True)
class ReportCreate:
    """ Report schema to retrieve from POST methods. """

    name: str
    bkgd_task_id: Union[Unset, int] = UNSET
    expires_on: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        bkgd_task_id = self.bkgd_task_id
        expires_on: Union[Unset, str] = UNSET
        if not isinstance(self.expires_on, Unset):
            expires_on = self.expires_on.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if bkgd_task_id is not UNSET:
            field_dict["bkgd_task_id"] = bkgd_task_id
        if expires_on is not UNSET:
            field_dict["expires_on"] = expires_on

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        bkgd_task_id = d.pop("bkgd_task_id", UNSET)

        expires_on: Union[Unset, datetime.datetime] = UNSET
        _expires_on = d.pop("expires_on", UNSET)
        if not isinstance(_expires_on, Unset):
            expires_on = isoparse(_expires_on)

        report_create = cls(
            name=name,
            bkgd_task_id=bkgd_task_id,
            expires_on=expires_on,
        )

        report_create.additional_properties = d
        return report_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class MetadataType(str, Enum):
    NUMBER = "number"
    BOOL = "bool"
    DATE = "date"
    DATERANGE = "daterange"
    STRING = "string"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PasswordPolicyUpdate")


@attr.s(auto_attribs=True)
class PasswordPolicyUpdate:
    """ Policy update schema. """

    min_length: Union[Unset, int] = 8
    expiration: Union[Unset, int] = 90
    max_similarity: Union[Unset, float] = 0.6
    max_login_attempts: Union[Unset, int] = 3
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        min_length = self.min_length
        expiration = self.expiration
        max_similarity = self.max_similarity
        max_login_attempts = self.max_login_attempts

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if min_length is not UNSET:
            field_dict["min_length"] = min_length
        if expiration is not UNSET:
            field_dict["expiration"] = expiration
        if max_similarity is not UNSET:
            field_dict["max_similarity"] = max_similarity
        if max_login_attempts is not UNSET:
            field_dict["max_login_attempts"] = max_login_attempts

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        min_length = d.pop("min_length", UNSET)

        expiration = d.pop("expiration", UNSET)

        max_similarity = d.pop("max_similarity", UNSET)

        max_login_attempts = d.pop("max_login_attempts", UNSET)

        password_policy_update = cls(
            min_length=min_length,
            expiration=expiration,
            max_similarity=max_similarity,
            max_login_attempts=max_login_attempts,
        )

        password_policy_update.additional_properties = d
        return password_policy_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
